import { ArrowLeftOutlined, ArrowRightOutlined } from "@ant-design/icons";
import {
  Button,
  Card,
  Col,
  ConfigProvider,
  Divider,
  Form,
  Grid,
  InputNumber,
  Row,
  Slider,
  Space,
  TimePicker,
} from "antd";
import { useForm } from "antd/es/form/Form";
import useMessage from "antd/es/message/useMessage";
import { useEffect, useRef, useState } from "react";
import { myFetch } from "./utils";
import dayjs from "dayjs";

const { useBreakpoint } = Grid;

const Constants = {
  DISTANCE: {
    MIN: 0,
    MAX: 100,
  },
  ACCELERATION: {
    MIN: 0,
    MAX: 100,
  },
  DURATION: {
    MIN: 1,
    MAX: 60,
  },
};

const PROGRES_BAR_INTERVAL_TIME = 20;

function setProgressBarTime(milliseconds) {
  const elem = document.getElementById("progress-bar-time");

  if (elem === null) return;

  if (milliseconds <= 0) {
    elem.innerText = "";
    return;
  }

  const seconds = Math.floor(milliseconds / 1000);
  const minutes = Math.floor(seconds / 60);
  const hours = Math.floor(minutes / 60);

  elem.innerText = `${hours > 0 ? hours + "h " : ""}${
    minutes > 0 ? (minutes % 60) + "m " : ""
  }${seconds % 60}s`;
}

function App() {
  const screens = useBreakpoint();
  const [form] = useForm();
  const [messageApi, messageContextHolder] = useMessage();

  const [distance, setDistance] = useState([20, 50]);
  const [acceleration, setAcceleration] = useState(0);
  const [direction, setDirection] = useState(0);

  const remainingTimeRef = useRef(null);
  const [processStatus, setProcessStatus] = useState("");

  const setRemainingTimeProgress = (time) => {
    remainingTimeRef.current = time * 1000;
  };

  const showErrorMessage = () => {
    messageApi.error({
      type: "error",
      content: "Ein Fehler ist aufgetreten",
    });
  };

  const handleResponse = (response) => {
    console.log(response);

    if (response.acceleration !== undefined)
      setAcceleration(response.acceleration);

    if (response.duration !== undefined) {
      const format = "HH:mm:ss";

      form.setFieldsValue({
        duration: dayjs(
          `${Math.floor(response.duration / 3600)}:${
            Math.floor(response.duration / 60) % 60
          }:${response.duration % 60}`,
          format
        ),
      });
    }

    if (response.distance !== undefined) setDistance(response.distance);
    if (response.direction !== undefined) setDirection(response.direction);
    if (response.runningUntil !== undefined)
      setRemainingTimeProgress(response.runningUntil);
    if (response.status !== undefined) setProcessStatus(response.status);
  };

  useEffect(() => {
    myFetch({
      url: "/status",
      method: "GET",
      showNotification: messageApi,
    })
      .then((response) => handleResponse(response))
      .catch((error) => {
        console.error(error);
        showErrorMessage();
      });
  }, []);

  useEffect(() => {
    if (remainingTimeRef.current === null) return;

    let elem = document.getElementById("progress-bar");
    let width = 1;
    let currentProgress = 0;

    if (elem === null) return;

    const intervalId = setInterval(frame, PROGRES_BAR_INTERVAL_TIME);

    function frame() {
      if (width >= 100) {
        clearInterval(intervalId);
        setRemainingTimeProgress(0);

        if (processStatus !== "Angehalten" && processStatus !== "")
          setProcessStatus("Fertig");
      } else {
        width = (100 / remainingTimeRef.current) * currentProgress;

        if (width > 100) width = 100;

        currentProgress += PROGRES_BAR_INTERVAL_TIME;
        elem.style.width = width + "%";
      }

      setProgressBarTime(remainingTimeRef.current - currentProgress);
    }

    return () => {
      clearInterval(intervalId);
    };
  }, [remainingTimeRef.current]);

  const sendControlRequest = (urlPath) => {
    form
      .validateFields()
      .then((values) => {
        setProcessStatus("Sende Anfrage");

        let durationInSeconds = 0;

        if (values.duration !== undefined) {
          const duration = values.duration;

          durationInSeconds =
            duration.$s + duration.$m * 60 + duration.$H * 3600;
        }

        myFetch({
          url: urlPath,
          method: "POST",
          body: {
            distance: distance,
            acceleration: acceleration,
            direction: direction,
            duration: durationInSeconds,
          },
          showNotification: messageApi,
        })
          .then((response) => handleResponse(response))
          .catch((error) => {
            console.error(error);
            showErrorMessage();
            setProcessStatus("");
            setRemainingTimeProgress(0);
          });
      })
      .catch((err) => console.error(err));
  };

  return (
    <div
      style={{
        display: "flex",
        flexDirection: "column",
        alignItems: "center",
        overflow: "auto",
        marginTop: 60,
        marginBottom: 60,
      }}
    >
      {messageContextHolder}

      <Card
        title="Kamera Slider"
        style={{
          width: "90%",
          maxWidth: 800,
        }}
      >
        <Form
          form={form}
          layout="vertical"
          onFinish={() => sendControlRequest("/start")}
          requiredMark={false}
        >
          <div
            style={{
              display: "flex",
              justifyContent: "space-between",
              paddingBottom: 4,
            }}
          >
            <span>{`Status${
              processStatus !== "" ? `: ${processStatus}` : ""
            }`}</span>

            <span
              id="progress-bar-time"
              style={{
                display: "inline-block",
                width: 100,
                textAlign: "right",
              }}
            />
          </div>

          <div
            style={{
              backgroundColor: "#f1f0f1",
              height: 8,
              borderRadius: 12,
            }}
          >
            <div
              id="progress-bar"
              style={{
                width: "0%",
                height: "100%",
                backgroundColor: "#98ccfc",
                borderRadius: 12,
              }}
            />
          </div>

          <Divider />

          <Form.Item label="Distanz">
            <Row>
              <Col xs={24} md={4}>
                <InputNumber
                  min={Constants.DISTANCE.MIN}
                  max={Constants.DISTANCE.MAX}
                  value={distance[0]}
                  step={1}
                  onChange={(value) =>
                    setDistance((newDistance) => {
                      const newData = [...newDistance];

                      newData[0] = value;

                      return newData;
                    })
                  }
                />
              </Col>

              <Col xs={24} md={16}>
                <Slider
                  range
                  defaultValue={[20, 50]}
                  marks={{ 0: "0", 100: "100" }}
                  min={Constants.DISTANCE.MIN}
                  max={Constants.DISTANCE.MAX}
                  value={distance}
                  onChange={(values) => setDistance(values)}
                />
              </Col>

              <Col xs={24} md={4}>
                <InputNumber
                  min={Constants.DISTANCE.MIN}
                  max={Constants.DISTANCE.MAX}
                  style={{
                    margin: screens.md ? "0 26px" : "",
                  }}
                  value={distance[1]}
                  step={1}
                  onChange={(value) =>
                    setDistance((newDistance) => {
                      const newData = [...newDistance];

                      newData[1] = value;

                      return newData;
                    })
                  }
                />
              </Col>
            </Row>
          </Form.Item>

          <Form.Item
            name="duration"
            label="Dauer"
            rules={[
              {
                required: true,
                message: "Bitte geben Sie die Dauer ein",
              },
            ]}
          >
            <TimePicker showNow={false} />
          </Form.Item>

          <Form.Item label="Beschleunigung">
            <Row>
              <Col xs={24} md={20}>
                <Slider
                  defaultValue={20}
                  marks={{ 0: "0", 100: "100" }}
                  min={Constants.ACCELERATION.MIN}
                  max={Constants.ACCELERATION.MAX}
                  value={acceleration}
                  onChange={(value) => setAcceleration(value)}
                />
              </Col>

              <Col xs={24} md={4}>
                <InputNumber
                  min={Constants.ACCELERATION.MIN}
                  max={Constants.ACCELERATION.MAX}
                  style={{
                    margin: screens.md ? "0 26px" : "",
                  }}
                  value={acceleration}
                  onChange={(value) => setAcceleration(value)}
                />
              </Col>
            </Row>
          </Form.Item>

          <Form.Item label="Richtung">
            <Space>
              <Button
                type={direction === 0 ? "primary" : "default"}
                shape="round"
                icon={<ArrowLeftOutlined />}
                onClick={() => setDirection(0)}
              >
                Links
              </Button>

              <Button
                type={direction === 1 ? "primary" : "default"}
                shape="round"
                icon={<ArrowRightOutlined />}
                iconPosition="end"
                onClick={() => setDirection(1)}
              >
                Rechts
              </Button>
            </Space>
          </Form.Item>

          <Form.Item>
            <Row gutter={[16, 16]}>
              <Col span={12}>
                <Button
                  type="primary"
                  block
                  danger
                  onClick={() => {
                    setProcessStatus("Sende Anfrage");

                    myFetch({
                      url: "/stop",
                      method: "POST",
                      showNotification: messageApi,
                    })
                      .then((response) => {
                        console.log(response);

                        setRemainingTimeProgress(0);
                        setProcessStatus("Angehalten");
                      })
                      .catch((error) => {
                        console.error(error);
                        showErrorMessage();
                        setProcessStatus("Angehalten");
                        setRemainingTimeProgress(0);
                      });
                  }}
                >
                  Stop
                </Button>
              </Col>

              <Col span={12}>
                <ConfigProvider
                  theme={{
                    token: {
                      colorPrimary: "#27ae60",
                    },
                  }}
                >
                  <Button type="primary" htmlType="submit" block>
                    Start
                  </Button>
                </ConfigProvider>
              </Col>
            </Row>
          </Form.Item>
        </Form>

        <Divider />

        <Button
          type="primary"
          block
          onClick={() => sendControlRequest("/moveNextPoint")}
        >
          Fahre zum nächsten Punkt
        </Button>
      </Card>
    </div>
  );
}

export default App;
